header {* Very Simple Propositional Logic II *}
theory VSPL
imports VSPL0
begin

text{*
  Hier wird eine Konstante \emph{definiert}, d.h. sie wird als
  abkürzende Schreibweise vereinbart:
  *}

definition
  Not    :: "o => o"                     ("~ _" [40] 40)
where
  "~P    == P --> False"


text{*
  Die Einführungs- und Eliminationsregel für Not werden als
  Theoreme \emph{bewiesen}. Im wesentlichen handelt es sich 
  um die Einführungs- und Eliminationsregel für --> mit 
  geeigneter Instanziierung für P (die hier Isabelle für uns
  erledigt).
  *}

lemma notE: "[| ~P; P |] ==> False"
  apply (unfold Not_def)
   --{* Mit unfold wird die linke Seite einer Definition durch 
        rechte Seite ersetzt; die Definition wird "aufgefaltet". *}
  apply (erule impE)
  apply (assumption)
  done

lemma notI: "(P ==> False) ==> ~P"
  apply (unfold Not_def)
  apply (rule impI)
  apply (assumption)
  done

text{*
  Hier wird das Lemma Raa' definiert als das Lemma Raa, in dem alle Vorkommen
  von P--> False durch ~P ersetzt werden ("einfalten" der Konstantendefinition).
  *}

lemmas Raa' = Raa [folded Not_def]
   and notnot  = doubleneg [folded Not_def]
   and notnot2 = doubleneg2 [folded Not_def]
 
text{*
  Definition der Disjunktion, sowie die Einführungs- und Elemininationsregeln.
  *}

definition
  Or  :: "[o, o] => o"                (infixr "|" 30)
where
  "P | Q == ~(~P & ~Q)"

theorem disjI1: "P ==> P | Q"
  apply (unfold Or_def)
  apply (rule notI)
  apply (rule notE [where P="P"])
  apply (erule conjE1) 
  apply (assumption)
  done

lemma disjI2: "Q ==> P | Q"
  apply (unfold Or_def)
  apply (rule notI)
  apply (rule notE [where P="Q"])
  apply (erule conjE2) 
  apply (assumption)
  done

lemma disjE: "[| P | Q; P==> R; Q ==> R |] ==> R"
  apply (unfold Or_def)
  apply (rule Raa')
  apply (rule notE [where P="~P & ~Q"])
  apply (assumption)
  apply (rule conjI)
  apply (rule notI)
  apply (rule notE [where P="R"])
  apply (assumption)
  apply (drule impI)
  apply (erule impE [where P="P"])
  apply (assumption)
  apply (rule notI)
  apply (rule notE [where P="R"])
  apply (assumption)
  apply (drule impI [where P="Q"])
  apply (rule impE [where P="Q"])
  apply (assumption, assumption)
  done

lemma tertiumNonDatur: "P | ~ P"
  apply (unfold Or_def)
  apply (rule notI)
  apply (rule notE [where P="~ P"])
  apply (drule conjE2, assumption)
  apply (drule conjE1, assumption) 
  done

text{*
  Definition der Äquivalenz (Biimplikation) mit seinen Regeln. 
  *}
definition
  Iff :: "[o, o] => o"               (infixr "<-->" 20)
where
  "P <--> Q == ((P --> Q) & (Q --> P))"

lemma iffI: "[| P--> Q; Q --> P |] ==> P <--> Q"
  apply (unfold Iff_def)
  apply (rule conjI)
  apply (assumption)
  apply (assumption)
  done

lemma iffE1: "[| P <--> Q |] ==> (P --> Q)"
  apply (unfold Iff_def)
  apply (erule conjE1)
  done

lemma iffE2: "[| P <--> Q |] ==> (Q --> P)"
  apply (unfold Iff_def)
  apply (erule conjE2)
  done

lemma iff_notnot: "~~ P <--> P"
  apply (rule iffI)
  apply (rule notnot)
  apply (rule notnot2)
  done

lemma imp_refl: "P --> P"
  apply (rule impI)
  apply (assumption)
  done


lemma iff_refl: "P <--> P"
  apply (rule iffI)
  apply (rule imp_refl)
  apply (rule imp_refl)
  done

end
