header{* Very Simple Presburger Arithmetic *}
theory VSPresburger
imports VSPredEq
begin

text{*
 Presburger Arithmetic: axiomising the natural numbers in five easy axioms.
 *}

typedecl
  N --{* The type of natural numbers *}

axiomatization
  Z :: "N"
and
  S :: "N => N"
where
  zero: "~ (S n = Z)"
and
  succ_inj: "[| S m = S n |] ==> m= n"
and
  induct: "[| P Z ;  ALL x. P x --> P (S x) |] ==> P y"

axiomatization
  plus :: "N=> N=> N"  (infix "+" 60)
where
  plus_zero_l: "Z+ x= x"
and
  plus_succ_l: "S x+ y= S(x+ y)"

section{* Einige Eigenschaften der Addition. *}

text{* Z ist ein neutrales Element. *}

lemma plus_zero_r: "x+ Z= x"
  apply (rule induct)
  apply (rule plus_zero_l)
  apply (rule allI, rule impI) 
  apply (rule trans)
  apply (rule plus_succ_l)
  apply (rule cong [where f="S"])
  apply (assumption)
  done

lemma plus_succ_r: "S(a+ b)= a+ S b"
  apply (rule induct [where y= "a"])
  --{* Induction base: *}
  apply (rule trans)
  apply (rule cong [where f="S"])
  apply (rule plus_zero_l)
  apply (rule sym)
  apply (rule plus_zero_l)
  --{* Induction step: *}
  apply (rule allI, rule impI)
  apply (rule trans)  
  apply (rule cong [where f="S"])
  apply (rule plus_succ_l)
  apply (rule trans)
  apply (rule cong [where f="S"])
  apply (assumption)
  apply (rule sym)
  apply (rule plus_succ_l)
  done

text{* Kommutativitt der Addition. *}

lemma plus_comm: "a+ b= b+ a"
  apply (rule induct [where y="a"])
  --{* Induction base: *}
  apply (rule trans)
  apply (rule plus_zero_l)
  apply (rule sym)
  apply (rule plus_zero_r)
  --{* Induction step: *}
  apply (rule allI, rule impI)
  apply (rule trans)
  apply (rule plus_succ_l)
  apply (rule trans)
  apply (rule cong [where f="S"])
  apply assumption 
  apply (rule plus_succ_r)
  done

text {* Assoziativitt der Addition. *}

lemma plus_assoc: "a+ (b+ c)= (a+ b)+ c"
  apply (rule induct [where y="a"])
  --{* Induction base *}
  apply (rule trans, rule plus_zero_l)
  apply (rule trans, rule cong [where f="%y. y+ c"])
  apply (rule sym, rule plus_zero_l, rule refl)
  --{* Induction step *}
  apply (rule allI, rule impI)
  apply (rule trans, rule plus_succ_l)
  apply (rule trans, rule cong [where f="%y. S y"])
  apply (assumption) --{* Induction hypothesis *} 
  apply (rule trans, rule sym, rule plus_succ_l)
  apply (rule trans, rule cong [where f="%y. y+ c"])
  apply (rule sym, rule plus_succ_l, rule refl)
  done

end
